/** PCA9955x constant current LED driver control library
 *
 *  @class   PCA995x
 *  @author  NXP Semiconductors Japan
 *  @version 0.7(07-Oct-2013)
 *
 *  Copyright: 2011-2013 NXP Semiconductors
 *  Released under the Achache2 License: http://mbed.org/license/mit
 *  This is a library that controls PCA9955 a constant current LED driver
 *  Example:
 *  @code
 *  #include    "mbed.h"
 *  #include    "PCA9955.h"
 *
 *  #define     MAX_IREF    0x20
 *
 *  PCA9955     led_driver( p28, p27, 0xC0 );  // making an instance of PCA9955
 *
 *  main()
 *  {
 *      char    v[ 16 ] = { 0xFF };
 *      int     sel;
 *
 *      srand( 0 );
 *
 *      led_driver  = 0xFFFF;
 *      led_driver.set_all_intensity( v );              //  PWM
 *      led_driver.set_all_intensity( MAX_IREF, true ); //  IREF
 *
 *      while ( 1 ) {
 *          for ( sel = 0; sel < 16; sel++ ) {
 *              v[ sel ]  = (char)((float)(v[ sel ]) * 0.8);
 *          }
 *
 *          v[ rand() % 16 ]  = 0xFF;
 *
 *          led_driver.set_all_intensity( v );              //  PWM
 *          wait( 0.05 );
 *      }
 *  }
 *  @endcode
 */

#include    "mbed.h"
#include    "PCA995x.h"

PCA995x::PCA995x(
    PinName     I2C_sda,
    PinName     I2C_scl,
    char        I2C_address, 
    char        device_type
) : i2c( I2C_sda, I2C_scl ), i2c_addr( I2C_address ), dev_type( device_type ) {

    i2c.frequency( 400 * 1000 );
    
    char    *init_array;
    int     array_size;
    int     i;
    
    array_size  = 1 + 5 + (dev_type / 4) + (dev_type * 2);
    
    if ( NULL == (init_array    = (char *)malloc( array_size * sizeof( char ) )) )
        error( "malloc error at device initialization" );
    
    init_array[ 0 ] = AUTO_INCREMENT | REGISTER_START;
    init_array[ 1 ] = 0x00;
    init_array[ 2 ] = 0x05;
     
    for ( i = 3; i < (dev_type / 4); i++ )  //  for LEDOUT registers
        init_array[ i ] = 0xAA;
        
    for ( ; i < dev_type; i++ )  //  for PWMn registers
        init_array[ i ] = 0x00;
        
    for ( ; i < dev_type; i++ )  //  for IREFn registers
        init_array[ i ] = 0x10;
        
    write( init_array, array_size );
}

void PCA995x::set_single_individual_intensity( char port, char val, int iref ) {
    write( (command_reg)((iref ? IREF_REGISTER_START : PWM_REGISTER_START) + port), val );
}

void PCA995x::set_all_intensity( char *p, int iref ) {
    char    pwm[ 1 + 24 ] = { AUTO_INCREMENT | (iref ? IREF_REGISTER_START : PWM_REGISTER_START) };

    for ( int i = 1; i <= dev_type; i++ )
        pwm[ i ]    = *p++;

    write( pwm, 1 + dev_type );
}

void PCA995x::set_all_intensity( char val, int iref ) {
    char    c[ dev_type ];

    for ( int i = 0; i < dev_type; i++ )
        c[ i ]    = val;

    set_all_intensity( c, iref );
}

void PCA995x::get_all_intensity( char *p, int iref ) {
    read( (command_reg)(AUTO_INCREMENT | (iref ? IREF_REGISTER_START : PWM_REGISTER_START)), p, dev_type );
}

PCA995x& PCA995x::operator= ( int c ) {
    char    a[ 1 + 6 ]  = { AUTO_INCREMENT | LEDOUT_REGISTER_START, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    char    out_val = 0x2;

    for ( int i = 0; i < dev_type; i++ )
        a[ (i / 4) + 1 ]  |= (((c >> i) & 0x1) ? out_val : 0x0) << ((i % 4) << 1);

    write( a, 1 + (dev_type / 4) );
    
    return *this;
}


void PCA995x::write( char *data, int length ) {
    i2c.write( i2c_addr, data, length );
}

void PCA995x::write( command_reg reg_addr, char data ) {
    char    c[2];

    c[0]    = reg_addr;
    c[1]    = data;

    i2c.write( i2c_addr, c, 2 );
}

#if 0
void PCA995x::write( int reg_addr, int data ) {
    write( (command_reg)reg_addr, (char)data );
}
#endif

void PCA995x::read( command_reg reg_addr, char *data, int length ) {
    i2c.write( i2c_addr, (char *)(&reg_addr), 1, true );
    i2c.read(  i2c_addr, data, length );
}

char PCA995x::read( command_reg reg_addr ) {
    i2c.write( i2c_addr, (char *)(&reg_addr), 1, true );
    i2c.read(  i2c_addr, (char *)(&reg_addr), 1 );

    return ( reg_addr );
}

#if 1
char PCA995x::read( int reg_addr ) {
    return ( read( (command_reg)reg_addr ) );
}
#endif


