/** PCA9955 constant current LED driver control library
 *
 *  @class   PCA9955
 *  @author  NXP Semiconductors Japan
 *  @version 0.5(15-Jun-2011) (publish 10-Aug-2012)
 *
 *  Copyright: 2011 NXP Semiconductors
 *  Released under the MIT License: http://mbed.org/license/mit
 *  This is a library that controls PCA9955 a constant current LED driver
 *  Example:
 *  @code
 *  #include    "mbed.h"
 *  #include    "PCA9955.h"
 *
 *  #define     MAX_IREF    0x20
 *
 *  PCA9955     led_driver( p28, p27, 0xC0 );  // making an instance of PCA9955
 *
 *  main()
 *  {
 *      char    v[ 16 ] = { 0xFF };
 *      int     sel;
 *
 *      srand( 0 );
 *
 *      led_driver  = 0xFFFF;
 *      led_driver.set_all_intensity( v );              //  PWM
 *      led_driver.set_all_intensity( MAX_IREF, true ); //  IREF
 *
 *      while ( 1 ) {
 *          for ( sel = 0; sel < 16; sel++ ) {
 *              v[ sel ]  = (char)((float)(v[ sel ]) * 0.8);
 *          }
 *
 *          v[ rand() % 16 ]  = 0xFF;
 *
 *          led_driver.set_all_intensity( v );              //  PWM
 *          wait( 0.05 );
 *      }
 *  }
 *  @endcode
 */

#ifndef     MBED_PCA995x
#define     MBED_PCA995x

#include    "mbed.h"



class PCA995x {
public:

    enum {
        PCA9955     = 16,
        PCA9956A    = 24,
    };

    enum command_reg {
        MODE1   = 0x00,
        MODE2,
        LEDOUT0,
        LEDOUT1,
        LEDOUT2,
        LEDOUT3,
        LEDOUT4,
        LEDOUT5,
        GRPPWM  = 0x08,
        GRPFREQ,
        PWM0    = 0x0A,
        PWM1,
        PWM2,
        PWM3,
        PWM4,
        PWM5,
        PWM6,
        PWM7,
        PWM8,
        PWM9,
        PWM10,
        PWM11,
        PWM12,
        PWM13,
        PWM14,
        PWM15,
        PWM16,
        PWM17,
        PWM18,
        PWM19,
        PWM20,
        PWM21,
        PWM22,
        PWM23,
        IREF0   = 0x22,
        IREF1,
        IREF2,
        IREF3,
        IREF4,
        IREF5,
        IREF6,
        IREF7,
        IREF8,
        IREF9,
        IREF10,
        IREF11,
        IREF12,
        IREF13,
        IREF14,
        IREF15,
        IREF16,
        IREF17,
        IREF18,
        IREF19,
        IREF20,
        IREF21,
        IREF22,
        IREF23,
        OFFSET  = 0x3A,
        SUBADR1,
        SUBADR2,
        SUBADR3,
        ALLCALLADR,

        REGISTER_START          = MODE1,
        LEDOUT_REGISTER_START   = LEDOUT0,
        PWM_REGISTER_START      = PWM0,
        IREF_REGISTER_START     = IREF0,
    };

    PCA995x( PinName I2C_sda, PinName I2C_scl, char I2C_address, char device_type );
    void set_single_individual_intensity( char port, char val, int iref = false );
    void set_all_intensity( char *p, int iref = false );
    void set_all_intensity( char val, int iref = false );
    void get_all_intensity( char *p, int iref = false );
    PCA995x& operator= ( int c );
//    unsigned short fault_test( void );
    void        write( char *data, int length );
    void        write( command_reg reg_addr, char data );
//    void        write( int reg_addr, int data );
    void        read( command_reg reg_addr, char *data, int length );
    char        read( command_reg reg_addr );
    char        read( int reg_addr );


private:

    enum {
        AUTO_INCREMENT          = 0x80,
    };


    I2C         i2c;
    char        i2c_addr;
    char        dev_type;
    static char init_array[];
}
;

#endif  //  MBED_PCA995x
